//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


function PhotoMapGUI(loc) {
   this.init(loc);
}

PhotoMapGUI.start = function() {
   var loc;
   if (window.arguments && window.arguments[0]) {
      loc = window.arguments[0];
   }
   PhotoMapGUI.instance = new PhotoMapGUI(loc);
};

PhotoMapGUI.stop = function() {
   if (PhotoMapGUI.instance) {
      PhotoMapGUI.instance.uninit();
      delete PhotoMapGUI.instance;
   }
};

PhotoMapGUI.prototype = {
   bigPhotos : false,

   // Initialisation and termination functions
   init : function(loc) {
      this.geocode         = document.getElementById("photomap.location.tb");
      this.mainPhotoHbox   = document.getElementById("photomap.photo.main.hbox");
      this.mainPhoto       = document.getElementById("photomap.photo.main");
      this.thumbs          = document.getElementById("photomap.photo.thumbs");
      this.mainEditPhoto   = document.getElementById("photomap.photo.edit.main");
      this.statusBar       = document.getElementById("photomap.statusbar.display");
      this.photoTabs       = document.getElementById("photomap.photo.tabs");
      this.photoFileName   = document.getElementById("photomap.photo.edit.file");
      this.photoEditLat    = document.getElementById("photomap.photo.edit.lat");
      this.photoEditLon    = document.getElementById("photomap.photo.edit.lon");
      this.photoEditType   = document.getElementById("photomap.photo.edit.zoom");
      this.photoEditTOD   = document.getElementById("photomap.photo.edit.time.of.day");
      this.photoEditPOV   = document.getElementById("photomap.photo.edit.pov");
      this.photoEditSeason     = document.getElementById("photomap.photo.edit.season");
      this.photoEditOtherTags  = document.getElementById("photomap.photo.edit.othertags");
      this.photoEditOutputTags = document.getElementById("photomap.photo.edit.outputtags");
      this.photoEditUploadButton = document.getElementById("photomap.photo.edit.upload");
      this.mode = "view";
      this.editMode = "add";

      this.controller = new PhotoMapController(this, loc);
   },
   
   uninit : function() {
      this.controller.shutdown();
   },

   homeAction : function() {
      this.controller.goHome();
   },

   forceRefreshAction : function() {
      this.controller.refresh();
   },

   helpAction : function() {
      this.controller.launchHelp();
   },

   optionsAction : function() {
      this.controller.launchOptions();
   },

   gotoPhotoAction : function() {
      this.controller.gotoPhoto(this.mainPhoto.photo);
   },

   getPhoto : function() {
      var photo;
      if (this.mainPhoto.photo) {
         photo = this.mainPhoto.photo;
      }
      return photo;
   },

   tabSelectAction : function() {
      var selectedTab = this.photoTabs.selectedIndex;
      if (selectedTab == 0) {
         this.mode = "view";
         this.controller.setPhotoViewMode();
      } else {
         this.mode = "edit";
         this.controller.setPhotoEditMode();
      }
   },

   setEditMode : function() {
      this.photoTabs.selectedIndex = 1;
   },

   latLonChangedAction : function() {  
      this.controller.setLocation({lat : this.photoEditLat.value, lon : this.photoEditLon.value});
      this.controller.updateTags();
   },

   thumbClickAction : function(photo) {
      if (photo) {
         this.setMainPhoto(photo);
      }
   },

   editDataChangedAction : function() {
      this.controller.updateTags();
   },

   getMapIFrame : function() {
      return document.getElementById("photomap.map.iframe");
   },

   setMapInfo : function(point, bounds, zoom) {
      this.statusBar.label = "Location: (" + point.lat + ", " + point.lon + ") Bounds: ((" + bounds.minY + ", " + bounds.minX + "), (" + bounds.maxY + ", " + bounds.maxX +")) Zoom: " + zoom;
      if (this.mode == "edit") {
         this.photoEditLat.value = point.lat;
         this.photoEditLon.value = point.lon;
         this.editDataChangedAction();
      }
   },

   setPhotos : function(imageSet) {
      var current = 1, image, docThumb, spacer, vbox;
      this.mainPhoto.setAttribute("src", "about:blank");
      if (this.mainPhoto.photo) {
         delete this.mainPhoto.photo;
      }
      while (this.thumbs.lastChild) {
         this.thumbs.removeChild(this.thumbs.lastChild);
      }
      
      if (imageSet) {
         for (img in imageSet) {
            image = imageSet[img];
            if (current == 1) {
               this.setMainPhoto(image);
            }
            docThumb = document.createElement("image");
            docThumb.setAttribute("id", "photomap.pictures.thumb" + current);
            docThumb.setAttribute("style", "border-bottom: 2px solid black; border-top: thin solid grey; border-left: thin solid grey; border-right: 2px solid black;");
            docThumb.setAttribute("onclick", "PhotoMapGUI.instance.thumbClickAction(this.photo);");
            docThumb.setAttribute("src", image.smallUrl);
            docThumb.photo = image;
            vbox = document.createElement("vbox");
            spacer = document.createElement("spacer");
            vbox.appendChild(docThumb);
            vbox.appendChild(spacer);
            this.thumbs.appendChild(vbox);
            spacer = document.createElement("spacer");
            spacer.setAttribute("style", "width: 2px");
            this.thumbs.appendChild(spacer);
            current++;
         }
      }
   },

   titleMaxLength : 25,

   setMainPhoto : function(photo) {
      if (photo) {
         if (this.bigPhotos) {
            this.mainPhoto.setAttribute("src", photo.regularUrl);
         } else {
            this.mainPhoto.setAttribute("src", photo.mediumUrl);
         }
         this.mainPhoto.photo = photo;
         if (photo.title.length > this.titleMaxLength) {
            document.getElementById("photomap.photo.main.title").value = photo.title.substring(0, this.titleMaxLength - 3) + "...";
         } else {
            document.getElementById("photomap.photo.main.title").value = photo.title;
         }
         document.getElementById("photomap.photo.main.pov").value   = "Facing: " + photo.pov;
         document.getElementById("photomap.photo.main.season").value = photo.season;
         document.getElementById("photomap.photo.main.tod").value    = photo.tod;
         document.getElementById("photomap.photo.main.lat").value    = "Lat: " + photo.lat;
         document.getElementById("photomap.photo.main.lon").value    = "Lon: " + photo.lon;

         document.getElementById("photomap.photo.main.goto").label = "Move to Picture";
         document.getElementById("photomap.photo.main.goto").removeAttribute("disabled");

         if (photo.url.indexOf("23hq") >= 0) {
            document.getElementById("photomap.photo.main.url").label = "See picture at 23";
         } else {
            document.getElementById("photomap.photo.main.url").label = "See picture at flickr";
         }
         document.getElementById("photomap.photo.main.url").tooltipText = photo.url;
         document.getElementById("photomap.photo.main.url").removeAttribute("disabled");
      } else {
         this.mainPhoto.setAttribute("src", "about:blank");
         this.mainPhoto.photo = null;
         document.getElementById("photomap.photo.main.title").value  = "";
         document.getElementById("photomap.photo.main.pov").value    = "";
         document.getElementById("photomap.photo.main.season").value = "";
         document.getElementById("photomap.photo.main.tod").value    = "";
         document.getElementById("photomap.photo.main.lat").value    = "";
         document.getElementById("photomap.photo.main.lon").value    = "";

         document.getElementById("photomap.photo.main.goto").label    = "";
         document.getElementById("photomap.photo.main.goto").setAttribute("disabled", "true");

         document.getElementById("photomap.photo.main.url").label    = "";
         document.getElementById("photomap.photo.main.url").tooltipText = "";
         document.getElementById("photomap.photo.main.url").setAttribute("disabled", "true");
      }
   },

   photoUrlAction : function() {
      this.openNewTab(this.mainPhoto.photo.url);
   },

   setEditOutputTags : function(tags) {
      this.photoEditOutputTags.value = tags;
   },

   getEditOtherTags : function() {
      return this.photoEditOtherTags.value;
   },

   getEditLat : function() {
      return this.photoEditLat.value;
   },

   getEditLon : function() {
      return this.photoEditLon.value;
   },

   getEditSeason : function() {
      return this.photoEditSeason.value;
   },

   getEditPOV : function() {
      return this.photoEditPOV.value;
   },

   getEditTOD : function() {
      return this.photoEditTOD.value;
   },

   getEditType : function() {
      return this.photoEditType.value;
   },

   newAccountAction : function() {
      // do this with a message bundle...
      var photoService = this.controller.getPhotoServiceName();
      window.alert("You are being redirected to the " + photoService + " site.\n\nAfter creating an account you must add at least 5 pictures\n before your account is reviewed and taken out of pending status.\n\nUntil your account is active, your pictures will not be visible\n in PhotoMap\n\nFor free accounts:\nonly the last 200 pictures will be visible in PhotoMap.");
      this.openWindow(this.controller.getNewAccountUrl());
   },

   authenticateAction : function() {
      // do this with a message bundle
      var photoService = this.controller.getPhotoServiceName();
      window.alert("You are being redirected to the " + photoService + " authentication page.\n\nWhen finished, sign out of flickr and return to PhotoMap.");
      this.controller.photoAuthenticate();
      window.alert("Press OK when authentication is done.");
      this.doneAuthenticateAction();
   },

   setAuthenticated : function() {
      document.getElementById("photomap.photo.edit.authenticate").setAttribute("disabled", "true");
      document.getElementById("photomap.photo.edit.info").setAttribute("collapsed", "false");

      // can't really do this until we have our picture...
      if (this.photoFileName.value) {
         document.getElementById("photomap.photo.edit.upload").removeAttribute("disabled");
      }
      document.getElementById("photomap.photo.edit.username").value = this.controller.getPhotoUserName();
      var total = this.controller.getPhotoTotalPictures();
      document.getElementById("photomap.photo.edit.totalPictures").value = total + " pictures";
      if (total > 175) {
         // highlight this some how
         // really this should be in the controller, where we know what the bad values are...
         // window.alert("Boy that's a lot of pictues...");
      }
   },

   setNeedsAuthentication : function(flag) {
      if (flag) {
         document.getElementById("photomap.photo.edit.info").setAttribute("collapsed", "true");
         document.getElementById("photomap.photo.edit.authenticate").removeAttribute("disabled");
         this.authenticated = false;
      } else {
         this.setAuthenticated();
         this.authenticated = true;
      }
   },

   doneAuthenticateAction : function() {
      this.controller.photoDoneAuthenticate();
   },

   uploadAction : function() {
      if (this.validateInput()) {
         if (this.editMode == "add") {
            this.controller.upload();
         } else if (this.editMode == "update") {
            this.controller.update();
         }
      }
   },

   validateInput : function() {
      // this shouldn't be here...
      var okay = true;
      if (this.getEditPOV() == "unknown") {
         okay = window.confirm("Point of view (north south etc.) is not set.\n\nContinue anyway?");
      }
      return okay;
   },

   doneUpload : function(wasError) {
      if (wasError) {
         window.alert("Problem with upload: " + wasError);
      } else {
         window.alert("Upload completed" );
         this.setPhotoFile("");
      }
   },

   doneUpdate : function(wasError) {
      if (wasError) {
         // need to translate not found into don't have permision
         window.alert("Problem with photo update: " + wasError);
      } else {
         window.alert("Update completed" );
      }
      this.setPhotoFile("");
   },

   // should this be trickier?
   openWindow : function(url) {
      window.open(url);
   },

   // should this be trickier?
   openNewTab : function(url) {
      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]. getService(Components.interfaces.nsIWindowMediator);
      var win = wm.getMostRecentWindow("navigator:browser");
      win.gBrowser.selectedTab = win.gBrowser.addTab(url);
   },

   logoutAction : function() {
      this.controller.photoLogout();
      this.setNeedsAuthentication(true);
   },

   editLoadAction : function() {
      this.controller.loadFile();
   },

   // enable/disable upload button here...
   setPhotoFile : function(filename) {
      this.photoFileName.value = filename;
      if (filename) {
         this.mainEditPhoto.setAttribute("src", "file://" + filename);
         if (this.authenticated) {
            document.getElementById("photomap.photo.edit.upload").removeAttribute("disabled");
         }
      } else {
         this.mainEditPhoto.setAttribute("src", "about:blank");
         document.getElementById("photomap.photo.edit.upload").setAttribute("disabled", "true");
      }
      document.getElementById("photomap.photo.edit.upload").label = "Upload";
      this.editMode = "add";
   },

   setHomeAction : function() {
      if (window.confirm("Set the current location as home?")) {
         this.controller.setHome();
      }
   },

   copyTagsAction : function() {
      this.controller.copyTags(this.photoEditOutputTags.value);
   },

   updateTransactionTotal : function(total) {
      document.getElementById("photomap.transactions").value = total;
   },

   linkAction : function() {
      this.controller.copyLinkText();
   },

   serverAction : function() {
      // launch the google earth server
      var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
      var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
      var topWindow = windowManagerInterface.getMostRecentWindow('global:PhotoMapServer');
   
      if (topWindow) {
         topWindow.focus();
      } else {
         window.openDialog('chrome://photomap/content/PhotoMapServer.xul', "_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar", this.controller);
      }
   },

   geocodeAction : function() {
      var location = this.geocode.value;
      this.controller.geocode(location);
   },

   setMainPhotoResizable : function() {
      this.mainPhoto.setAttribute("flex", "2");
   },

   setBigPhotos : function() {
      this.bigPhotos = true;
   },

   // drag and drop support
   onDragOver : function(event, flavour, session) {
      // we do nothing, but needs to be defined, because we accept stuff
   },

   onDragExit : function(event, session) {
      // we do nothing
   },

   onDrop : function(event, dropData, session) {
      if (dropData.flavour.contentType == "text/unicode") {
         this.controller.photoLocationDropped(dropData.data);
      } else if (dropData.flavour.contentType == "application/x-moz-file") {
         this.controller.fileDropped(dropData.data);
      }
   },

   getSupportedFlavours : function() {
      // what do I really want to support?
      var flavours = new FlavourSet();
      flavours.appendFlavour("text/unicode");
      flavours.appendFlavour("application/x-moz-file", "nsIFile");
      return flavours;
   },

   photoUpdate : function(photo) {
      this.setPhotoFile("");
      this.mainEditPhoto.setAttribute("src", photo.mediumUrl);
      this.photoEditType.value = photo.type;
      this.photoEditTOD.value = photo.tod;
      this.photoEditPOV.value = photo.pov;
      this.photoEditSeason.value = photo.season;
      // don't really want all of those other tags
      // this.photoEditOtherTags.value = photo.other;
      if (this.authenticated) {
         document.getElementById("photomap.photo.edit.upload").removeAttribute("disabled");
         document.getElementById("photomap.photo.edit.upload").label = "Add Tags";
         this.editMode = "update";
      }

      if (photo.lat != "0.0" || photo.lon != "0.0") {
         window.alert("If you want to reposition the picture, you should use " + photo.provider + " to delete the existing geotags first");
      }
   },

   problemWithPhotoDrop : function(e) {
      window.alert("There was a problem with what you dropped:" + e);
   }
};

